/****************************************************************************
* Simon for Nintendo Wii / Gamecube
* Version: 0.2
*
* Author:  teknecal
* Created: 30/05/2008
* Last Modified: 05/06/2008
* 
****************************************************************************/
#include <gccore.h>
#include <ogcsys.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <ogc/pad.h>
#include <ogc/system.h>
#include <unistd.h>

#include "main.h"

// Graphics
#include "simon.h"
#include "simonblue.h"
#include "simongreen.h"
#include "simonred.h"
#include "simonyellow.h"
#include "simontext.h"
#include "starttext.h"
#include "blanktext.h"
#include "youwin.h"
#include "youlost.h"

#define SEQ_LENGTH 15


// 2D Video Globals
static u32 *xfb;
static GXRModeObj *rmode;

// Vars
char seq [SEQ_LENGTH];
char color [4] = "0123";
int seqnum, seqlevel, userseqnum;
bool userinput, correctinput, ingame;

void Initialise() {
  
	VIDEO_Init();
	PAD_Init();
 
	switch(VIDEO_GetCurrentTvMode()) {
		case VI_NTSC:
			rmode = &TVNtsc480IntDf;
			break;
		case VI_PAL:
			rmode = &TVPal528IntDf;
			break;
		case VI_MPAL:
			rmode = &TVMpal480IntDf;
			break;
		default:
			rmode = &TVNtsc480IntDf;
			break;
	}

	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
 
	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

}

void SetScreen() {
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_Flush();
	VIDEO_WaitVSync();
}

void drawpic(int x,int y, int height, int width, const unsigned int data[]) {
	int i = 0,j = 0;
	for(i=0;i<=width;i++)
		for(j=0;j<=height-1;j++)
			xfb[(i+x)+320*(j+16+y)]=data[i+width*j];
}


// Animate the "spinning" lights on simon
void animate_simon() {
	int count = 0;
	int multipler = 0;
	bool running = true;
	u32 button = 0;
 
	drawpic(96, 410, 27, 120, starttext);
 
	while (running == true) { 
		if (count == 0) { 
			drawpic(60, 60, 336, 192, simonred); 
		}
		if (count == 1) { 
			drawpic(60, 60, 336, 192, simongreen); 
		}
		if (count == 2) { 
			drawpic(60, 60, 336, 192, simonyellow); 
		}		
		if (count == 3) { 
			drawpic(60, 60, 336, 192, simonblue); 
			count = -1; 
		}
		
		count++;
		
		// Sleep a little bit in the for loop so we can grab user input
		int i = 0;
		for (i = 0; i< 20; i++) {
			usleep(25000 - (multipler * 100));	
			PAD_ScanPads();
			button = PAD_ButtonsDown(0);
			if (button == PAD_BUTTON_A) {
				running = false;		
			}
			// Go faster or slower according to user input
			else if ((PAD_StickY(0) > 18 || PAD_SubStickY(0) > 18) && multipler <= 245) {
				multipler++;		
			}
			else if ((PAD_StickY(0) < -18 || PAD_SubStickY(0) < -18) && multipler >= 1) {
				multipler--;
			}
		}
		
		// Handle reboot/poweroff
		if (event == 1) {
			printf("POWER!\n");
			reload();
		} else if (event == 2) {
			printf("RESET!\n");
			reload();
		}
		if (button == PAD_BUTTON_START)
			reload();
		
		//printf("multipler = %i \n",multipler);
		
		SetScreen();
	}
}

// Reset the variables and text
void restart_game() {
	seqnum = 0;
	seqlevel = 0;
	userseqnum = 0; 
	userinput = false;
	correctinput = true;
	ingame = true;
	
	drawpic(96, 410, 28, 120, blanktext);
	drawpic(60, 60, 336, 192, simon);

	// Randomise the sequence
	srand((unsigned)time(0)); 
 
	int q;
	for(q=0; q<SEQ_LENGTH; q++){ 
		char random_integer = '0' + rand() % 4;
		seq[q] = random_integer;
		//printf("%c \n", random_integer); 
	}
	printf("Sequence is %s \n",seq);
}


void game() {
	u32 button = 0;
 
	while (1) {
		while (ingame == true) {
			PAD_ScanPads();
			button = PAD_ButtonsDown(0);
			
			// If we are waiting for user input
			if (userinput == TRUE) {
				if (button == PAD_BUTTON_Y) {
					drawpic(60, 60, 336, 192, simonred);
					if (color[0] != seq[userseqnum]) {
						correctinput = false;
					}
				}
				if (button == PAD_BUTTON_X) {
					drawpic(60, 60, 336, 192, simongreen);
					if (color[1] != seq[userseqnum]) {
						correctinput = false;
					}
				}
				if (button == PAD_BUTTON_B) {
					drawpic(60, 60, 336, 192, simonblue);
					if (color[2] != seq[userseqnum]) {
						correctinput = false;
					}
				}
				if (button == PAD_BUTTON_A) {
					drawpic(60, 60, 336, 192, simonyellow);
					if (color[3] != seq[userseqnum]) {
						correctinput = false;
					}
				}
				
				if (button == PAD_BUTTON_A || button == PAD_BUTTON_B || button == PAD_BUTTON_X || button == PAD_BUTTON_Y) { 
					
					usleep(500000);
					
					drawpic(60,60, 336, 192, simon);
					
					
					//printf("userseq %i seqlevel %i \n",userseqnum, seqlevel);
					
					// If they entered the correct sequence
					if (correctinput == TRUE) {
						
						// Increment the users seq number
						if (userseqnum < seqlevel) {
							userseqnum++;
						}
						
						// If user has matched the seq level, increase the seq level and show the new sequence to the user
						else if (userseqnum >= seqlevel) {
							userseqnum = 0;
							seqlevel++;
							userinput = false;
							
							// Wait a little while
							usleep(500000);
							
							// If user has won
							if (seqlevel == SEQ_LENGTH) {
								//printf("You win! \n");
								drawpic(96, 410, 27, 120, youwin);
								
								// Blink the winning square
								int a;
								for (a = 0; a < 10; a++) {
									if (seq[SEQ_LENGTH-1] == '0') {
										drawpic(60, 60, 336, 192, simonred);
									}
									if (seq[SEQ_LENGTH-1] == '1') {
										drawpic(60, 60, 336, 192, simongreen);
									}
									if (seq[SEQ_LENGTH-1] == '2') {
										drawpic(60, 60, 336, 192, simonblue);
									}
									if (seq[SEQ_LENGTH-1] == '3') {
										drawpic(60, 60, 336, 192, simonyellow);
									}
									usleep(100000);
									SetScreen();
									
									drawpic(60, 60, 336, 192, simon);
									usleep(100000);
									SetScreen();
								}
								
								drawpic(96, 410, 28, 120, blanktext);
								usleep(500000);
								SetScreen();
								
								//printf("Press A to play again \n");
								ingame = false;
							}
						}
					}
					
					// Incorrect sequence
					else if (correctinput == FALSE) {
						//printf("Sorry wrong key.... \n");
						usleep(1000000);
						
						//printf("Right key was %c \n",seq[userseqnum]);
						//printf("You lost! \n");
						
						if (seq[userseqnum] == '0') {
							drawpic(60, 60, 336, 192, simonred);
						}
						if (seq[userseqnum] == '1') {
							drawpic(60, 60, 336, 192, simongreen);
						}
						if (seq[userseqnum] == '2') {
							drawpic(60, 60, 336, 192, simonblue);
						}
						if (seq[userseqnum] == '3') {
							drawpic(60, 60, 336, 192, simonyellow);
						}
						
						drawpic(96, 410, 27, 120, youlost);
						
						usleep(15000000);
						SetScreen();
						
						drawpic(60, 60, 336, 192, simon);
						drawpic(96, 410, 28, 118, blanktext);
						
						usleep(500000);
						SetScreen();
						
						ingame = false;
					}
				}
			}
			
			// If we are playing the sequence to the user
			if (userinput == FALSE) {
				usleep(1000000 - (seqlevel * 50000)); // Lower time every incrementation
				
				if (seq[seqnum] == color[0]) {
					drawpic(60, 60, 336, 192, simonred);
				}
				if (seq[seqnum] == color[1]) {
					drawpic(60, 60, 336, 192, simongreen);
				}
				if (seq[seqnum] == color[2]) {
					drawpic(60, 60, 336, 192, simonblue);
				}
				if (seq[seqnum] == color[3]) {
					drawpic(60, 60, 336, 192, simonyellow);
				}				
				
				if (seqnum == seqlevel) { 
					userinput = TRUE;
					seqnum = -1;
				}
				
				
				// Sleep a while and then show blank simon
				usleep(1000000 - (seqlevel * 50000));
				drawpic(60, 60, 336, 192, simon);
				
				seqnum++;  
			}
			
			// Handle reboot/poweroff
			if (event == 1) {
				printf("POWER!\n");
				reload();
			} else if (event == 2) {
				printf("RESET!\n");
				reload();
			}
			if (button == PAD_BUTTON_START)
				reload();
			
			SetScreen();
		}
		
		// First run or Game over? Then go back to menu
		animate_simon();
		restart_game();
	}
}

int main () {
 
	Initialise();
	PAD_Init();
	SYS_SetResetCallback (reset_cb);
 
	drawpic(93, 0, 48, 120, simontext);
 
	game();
 
	return 0;
}
